/********************************************************************** 
* test-file for simple embedded-calls.
* this includes a "main" function: C is 'on top' in the program.
**********************************************************************/

#include "pyembed.h"
#define TRACE(msg) printf("\n%s\n", msg);
char *getprogramname() { return "test"; }    /* not always needed */


main(argc, argv)
int argc;
char **argv;
{
    printf("Hello from C.\n");
    Py_Initialize();              /* initialize embedded python libraries */
    PySys_SetArgv(argc, argv);    /* optional: export command args to python */
    BASIC_EMBEDDING();
    EMBEDDED_CALL_API();
    printf("Bye from C.\n");
    Py_Exit(0);                   /* optional: runs sys.exitfunc handler */
}


BASIC_EMBEDDING()         /* test low-level interfaces */
{
    PyObject *module, *dict, *pstack;

    /********************************************************************
     * run code-strings in default namespace (module '__main__'); 
     * uses embedding + extending: os/sys use C extension modules;
     * can call PyRun_SimpleString, PyRun_String, PyEval_CallObject..
     ********************************************************************/

    TRACE("SIMPLE STRINGS")
    PyRun_SimpleString("import sys, os");
    PyRun_SimpleString("print 'Hello,', os.environ['USER'] + '.'"); 
    PyRun_SimpleString("print 'Args: ', sys.argv");
    PyRun_SimpleString("print 'Path: ', sys.path[0:2], '...'");

    /********************************************************************
     * use the stack-type extension in embedded code; PyRun_String uses
     * explicit name-spaces and parse-modes; the PyRun_String results 
     * are None, but they should really be DECREF'd  and return values 
     * should be checked for NULL (a python error/exception occurred);
     ********************************************************************/

    TRACE("STACK EXTENSION")
    module = PyImport_AddModule("__main__");             /* fetch a module */
    dict   = PyModule_GetDict(module);                   /* get its __dict__ */
    PyRun_String("from stacktype import *", file_input, dict, dict);
    PyRun_String("x = Stack()", file_input, dict, dict);
    PyRun_String("for c in 'SPAM':\n"                         /* multi-line */
                   "\tx.push(c)\n", file_input, dict, dict);  /* stmts okay */
    PyRun_String("print x, len(x)", file_input, dict, dict);     

    /* make a new instance */              
    pstack = PyRun_String("Stack()", eval_input, dict, dict); 
    PyObject_Print(pstack, stdout, 0);
    Py_DECREF(pstack);
}


EMBEDDED_CALL_API()                  /* test enhanced wrappers */
{
    int i, status, result;
    char *strres;                    /* actual text is stored in python */ 
    PyObject *pobject1, *pobject2;   /* raw python objects: 'O' output codes */
    
    /********************************************************************
     * use the run_codestr wrapper like PyRun_SimpleString:
     * run string in __main__ (NULL 1), with no result (NULL 2);
     * see order1.c (ahead) for dummy-module creation/use; 
     ********************************************************************/

    Run_Codestr(PY_STATEMENT, "print 'Hello api world'", NULL, "", NULL);

    /********************************************************************
     * turn on dynamic reload-on-call mode, and call a function in a 
     * module file; the file can be changed between the five calls here;
     * calls "main.func(4, 8)": see mainc.py for equivalent Python code;
     ********************************************************************/

    TRACE("DYNAMIC RELOADING")
    PY_RELOAD = 1;
    for (i=0; i < 5; i++) {
        status = Run_Function("main", "func", "i", &result, "(ii)", 4, 8);
        if (status == -1) {
            printf("Error running func\n");
            PyErr_Print();                       /* python stack dump */
        }
        else
            printf("result => %d\n", result);
        if (i < 4) {
            printf("change main.py now...");     /* edit in another window */
            getchar();                           /* wait for a keypress */
        }
    }

    /********************************************************************
     * turn on dynamic debugging, and run a string and a function; 
     * pdb stops execution as soon as the embedded code starts: set 
     * breakponts, step through code, etc.; both run "main.func(4, 8)";
     ********************************************************************/

    TRACE("DYNAMIC DEBUGGING")
    PY_RELOAD = 0;
    PY_DEBUG  = 1;       /* could do both at once */

    status = Run_Codestr(PY_EXPRESSION, "func(4, 8)", "main", "i", &result);
    printf("%d, %d\n\n", status, result);

    status = Run_Function("main", "func", "i", &result, "(ii)", 4, 8);
    printf("%d, %d\n", status, result);
    PY_DEBUG = 0;

    /********************************************************************
     * a few attribute tests: sys module, already imported in __main__;
     * in general, return values should be checked for errors too;
     ********************************************************************/
 
    TRACE("OBJECT ATTRIBUTES")
    Run_Command_Line("check sys");                            /* in __main__ */
    Run_Codestr(PY_EXPRESSION, "sys", NULL, "O", &pobject1);  /* __main__.sys */
    Get_Member(pobject1, "version", "s", &strres);            /* sys.version */
    printf("version => %s...\n", strres);

    Set_Member(pobject1, "version", "s", "1.4 1996?");        /* change it */
    Run_Codestr(PY_STATEMENT, "print sys.version", NULL, "", NULL);

    Get_Member(pobject1, "modules", "O", &pobject2);          /* sys.modules */
    Run_Method(pobject2, "has_key", "i", &result, "(s)", "main"); /* .has_key*/
    printf("result => %d\n", result);
    Py_XDECREF(pobject2);

    Get_Member(pobject1, "stdout", "O", &pobject2);           /* sys.stdout */
    Run_Method(pobject2, "write", "", NULL, "(s)", "right...\n");  /* .write*/
   
    Py_XDECREF(pobject1);   /* need to free raw python objects */
    Py_XDECREF(pobject2);   /* passed out with "O" format code */
}
